import { withTheme } from '@rjsf/core';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { TranslatableString, getUiOptions, getTemplate, getInputProps, ariaDescribedByIds, examplesId, errorId, helpId, titleId, descriptionId, canExpand, getSubmitButtonOptions, ADDITIONAL_PROPERTY_FLAG, schemaRequiresTrueValue, labelValue, enumOptionsIsSelected, optionId, enumOptionsSelectValue, enumOptionsDeselectValue, enumOptionsValueForIndex, enumOptionsIndexForValue } from '@rjsf/utils';
import Button from 'react-bootstrap/Button';
import { BsPlus } from '@react-icons/all-files/bs/BsPlus';
import Row from 'react-bootstrap/Row';
import Col from 'react-bootstrap/Col';
import Container from 'react-bootstrap/Container';
import Form$1 from 'react-bootstrap/Form';
import Card from 'react-bootstrap/Card';
import ListGroup from 'react-bootstrap/ListGroup';
import { IoIosCopy } from '@react-icons/all-files/io/IoIosCopy';
import { IoIosRemove } from '@react-icons/all-files/io/IoIosRemove';
import { AiOutlineArrowUp } from '@react-icons/all-files/ai/AiOutlineArrowUp';
import { AiOutlineArrowDown } from '@react-icons/all-files/ai/AiOutlineArrowDown';
import FormControl from 'react-bootstrap/FormControl';
import InputGroup from 'react-bootstrap/InputGroup';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;
  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }
  return target;
}

var _excluded$2 = ["uiSchema", "registry"];
function AddButton(_ref) {
  var registry = _ref.registry,
    props = _objectWithoutPropertiesLoose(_ref, _excluded$2);
  var translateString = registry.translateString;
  return jsx(Button, _extends({}, props, {
    style: {
      width: '100%'
    },
    className: "ml-1 " + props.className,
    title: translateString(TranslatableString.AddItemButton),
    children: jsx(BsPlus, {})
  }));
}

function ArrayFieldItemTemplate(props) {
  var children = props.children,
    disabled = props.disabled,
    hasToolbar = props.hasToolbar,
    hasCopy = props.hasCopy,
    hasMoveDown = props.hasMoveDown,
    hasMoveUp = props.hasMoveUp,
    hasRemove = props.hasRemove,
    index = props.index,
    onCopyIndexClick = props.onCopyIndexClick,
    onDropIndexClick = props.onDropIndexClick,
    onReorderClick = props.onReorderClick,
    readonly = props.readonly,
    registry = props.registry,
    uiSchema = props.uiSchema;
  var _registry$templates$B = registry.templates.ButtonTemplates,
    CopyButton = _registry$templates$B.CopyButton,
    MoveDownButton = _registry$templates$B.MoveDownButton,
    MoveUpButton = _registry$templates$B.MoveUpButton,
    RemoveButton = _registry$templates$B.RemoveButton;
  var btnStyle = {
    flex: 1,
    paddingLeft: 6,
    paddingRight: 6,
    fontWeight: 'bold'
  };
  return jsx("div", {
    children: jsxs(Row, {
      className: 'mb-2  d-flex align-items-center',
      children: [jsx(Col, {
        xs: '9',
        lg: '9',
        children: children
      }), jsx(Col, {
        xs: '3',
        lg: '3',
        className: 'py-4',
        children: hasToolbar && jsxs("div", {
          className: 'd-flex flex-row',
          children: [(hasMoveUp || hasMoveDown) && jsx("div", {
            className: 'm-0 p-0',
            children: jsx(MoveUpButton, {
              className: 'array-item-move-up',
              style: btnStyle,
              disabled: disabled || readonly || !hasMoveUp,
              onClick: onReorderClick(index, index - 1),
              uiSchema: uiSchema,
              registry: registry
            })
          }), (hasMoveUp || hasMoveDown) && jsx("div", {
            className: 'm-0 p-0',
            children: jsx(MoveDownButton, {
              style: btnStyle,
              disabled: disabled || readonly || !hasMoveDown,
              onClick: onReorderClick(index, index + 1),
              uiSchema: uiSchema,
              registry: registry
            })
          }), hasCopy && jsx("div", {
            className: 'm-0 p-0',
            children: jsx(CopyButton, {
              style: btnStyle,
              disabled: disabled || readonly,
              onClick: onCopyIndexClick(index),
              uiSchema: uiSchema,
              registry: registry
            })
          }), hasRemove && jsx("div", {
            className: 'm-0 p-0',
            children: jsx(RemoveButton, {
              style: btnStyle,
              disabled: disabled || readonly,
              onClick: onDropIndexClick(index),
              uiSchema: uiSchema,
              registry: registry
            })
          })]
        })
      })]
    })
  });
}

var _excluded$1 = ["key"];
function ArrayFieldTemplate(props) {
  var canAdd = props.canAdd,
    disabled = props.disabled,
    idSchema = props.idSchema,
    uiSchema = props.uiSchema,
    items = props.items,
    onAddClick = props.onAddClick,
    readonly = props.readonly,
    registry = props.registry,
    required = props.required,
    schema = props.schema,
    title = props.title;
  var uiOptions = getUiOptions(uiSchema);
  var ArrayFieldDescriptionTemplate = getTemplate('ArrayFieldDescriptionTemplate', registry, uiOptions);
  var ArrayFieldItemTemplate = getTemplate('ArrayFieldItemTemplate', registry, uiOptions);
  var ArrayFieldTitleTemplate = getTemplate('ArrayFieldTitleTemplate', registry, uiOptions);
  // Button templates are not overridden in the uiSchema
  var AddButton = registry.templates.ButtonTemplates.AddButton;
  return jsx("div", {
    children: jsx(Row, {
      className: 'p-0 m-0',
      children: jsxs(Col, {
        className: 'p-0 m-0',
        children: [jsx(ArrayFieldTitleTemplate, {
          idSchema: idSchema,
          title: uiOptions.title || title,
          schema: schema,
          uiSchema: uiSchema,
          required: required,
          registry: registry
        }), jsx(ArrayFieldDescriptionTemplate, {
          idSchema: idSchema,
          description: uiOptions.description || schema.description,
          schema: schema,
          uiSchema: uiSchema,
          registry: registry
        }), jsxs(Container, {
          fluid: true,
          className: 'p-0 m-0',
          children: [items && items.map(function (_ref) {
            var key = _ref.key,
              itemProps = _objectWithoutPropertiesLoose(_ref, _excluded$1);
            return jsx(ArrayFieldItemTemplate, _extends({}, itemProps), key);
          }), canAdd && jsx(Container, {
            className: '',
            children: jsxs(Row, {
              className: 'mt-2',
              children: [jsx(Col, {
                xs: 9
              }), jsx(Col, {
                xs: 3,
                className: 'py-4 col-lg-3 col-3',
                children: jsx(AddButton, {
                  className: 'array-item-add',
                  onClick: onAddClick,
                  disabled: disabled || readonly,
                  uiSchema: uiSchema,
                  registry: registry
                })
              })]
            })
          })]
        }, "array-item-list-" + idSchema.$id)]
      })
    })
  });
}

function BaseInputTemplate(_ref) {
  var id = _ref.id,
    placeholder = _ref.placeholder,
    required = _ref.required,
    readonly = _ref.readonly,
    disabled = _ref.disabled,
    type = _ref.type,
    value = _ref.value,
    onChange = _ref.onChange,
    onChangeOverride = _ref.onChangeOverride,
    onBlur = _ref.onBlur,
    onFocus = _ref.onFocus,
    autofocus = _ref.autofocus,
    options = _ref.options,
    schema = _ref.schema,
    _ref$rawErrors = _ref.rawErrors,
    rawErrors = _ref$rawErrors === void 0 ? [] : _ref$rawErrors,
    children = _ref.children,
    extraProps = _ref.extraProps;
  var inputProps = _extends({}, extraProps, getInputProps(schema, type, options));
  var _onChange = function _onChange(_ref2) {
    var value = _ref2.target.value;
    return onChange(value === '' ? options.emptyValue : value);
  };
  var _onBlur = function _onBlur(_ref3) {
    var value = _ref3.target.value;
    return onBlur(id, value);
  };
  var _onFocus = function _onFocus(_ref4) {
    var value = _ref4.target.value;
    return onFocus(id, value);
  };
  // const classNames = [rawErrors.length > 0 ? "is-invalid" : "", type === 'file' ? 'custom-file-label': ""]
  return jsxs(Fragment, {
    children: [jsx(Form$1.Control, _extends({
      id: id,
      name: id,
      placeholder: placeholder,
      autoFocus: autofocus,
      required: required,
      disabled: disabled,
      readOnly: readonly,
      className: rawErrors.length > 0 ? 'is-invalid' : '',
      list: schema.examples ? examplesId(id) : undefined
    }, inputProps, {
      value: value || value === 0 ? value : '',
      onChange: onChangeOverride || _onChange,
      onBlur: _onBlur,
      onFocus: _onFocus,
      "aria-describedby": ariaDescribedByIds(id, !!schema.examples)
    })), children, Array.isArray(schema.examples) ? jsx("datalist", {
      id: examplesId(id),
      children: schema.examples.concat(schema["default"] && !schema.examples.includes(schema["default"]) ? [schema["default"]] : []).map(function (example) {
        return jsx("option", {
          value: example
        }, example);
      })
    }) : null]
  });
}

function DescriptionField(_ref) {
  var id = _ref.id,
    description = _ref.description;
  if (description) {
    return jsx("div", {
      children: jsx("div", {
        id: id,
        className: 'mb-3',
        children: description
      })
    });
  }
  return null;
}

function ErrorList(_ref) {
  var errors = _ref.errors,
    registry = _ref.registry;
  var translateString = registry.translateString;
  return jsxs(Card, {
    border: 'danger',
    className: 'mb-4',
    children: [jsx(Card.Header, {
      className: 'alert-danger',
      children: translateString(TranslatableString.ErrorsLabel)
    }), jsx(Card.Body, {
      className: 'p-0',
      children: jsx(ListGroup, {
        children: errors.map(function (error, i) {
          return jsx(ListGroup.Item, {
            className: 'border-0',
            children: jsx("span", {
              children: error.stack
            })
          }, i);
        })
      })
    })]
  });
}

var _excluded = ["icon", "iconType", "className", "uiSchema", "registry"];
function IconButton(props) {
  var icon = props.icon,
    iconType = props.iconType,
    otherProps = _objectWithoutPropertiesLoose(props, _excluded);
  return jsx(Button, _extends({
    block: iconType === 'block'
  }, otherProps, {
    variant: props.variant || 'light',
    size: 'sm',
    children: icon
  }));
}
function CopyButton(props) {
  var translateString = props.registry.translateString;
  return jsx(IconButton, _extends({
    title: translateString(TranslatableString.CopyButton)
  }, props, {
    icon: jsx(IoIosCopy, {})
  }));
}
function MoveDownButton(props) {
  var translateString = props.registry.translateString;
  return jsx(IconButton, _extends({
    title: translateString(TranslatableString.MoveDownButton)
  }, props, {
    icon: jsx(AiOutlineArrowDown, {})
  }));
}
function MoveUpButton(props) {
  var translateString = props.registry.translateString;
  return jsx(IconButton, _extends({
    title: translateString(TranslatableString.MoveUpButton)
  }, props, {
    icon: jsx(AiOutlineArrowUp, {})
  }));
}
function RemoveButton(props) {
  var translateString = props.registry.translateString;
  return jsx(IconButton, _extends({
    title: translateString(TranslatableString.RemoveButton)
  }, props, {
    variant: 'danger',
    icon: jsx(IoIosRemove, {})
  }));
}

/** The `FieldErrorTemplate` component renders the errors local to the particular field
 *
 * @param props - The `FieldErrorProps` for the errors being rendered
 */
function FieldErrorTemplate(props) {
  var _props$errors = props.errors,
    errors = _props$errors === void 0 ? [] : _props$errors,
    idSchema = props.idSchema;
  if (errors.length === 0) {
    return null;
  }
  var id = errorId(idSchema);
  return jsx(ListGroup, {
    as: 'ul',
    id: id,
    children: errors.map(function (error, i) {
      return jsx(ListGroup.Item, {
        as: 'li',
        className: 'border-0 m-0 p-0',
        children: jsx("small", {
          className: 'm-0 text-danger',
          children: error
        })
      }, i);
    })
  });
}

/** The `FieldHelpTemplate` component renders any help desired for a field
 *
 * @param props - The `FieldHelpProps` to be rendered
 */
function FieldHelpTemplate(props) {
  var idSchema = props.idSchema,
    help = props.help,
    hasErrors = props.hasErrors;
  if (!help) {
    return null;
  }
  var id = helpId(idSchema);
  return jsx(Form$1.Text, {
    className: hasErrors ? 'text-danger' : 'text-muted',
    id: id,
    children: help
  });
}

function FieldTemplate(_ref) {
  var id = _ref.id,
    children = _ref.children,
    displayLabel = _ref.displayLabel,
    _ref$rawErrors = _ref.rawErrors,
    rawErrors = _ref$rawErrors === void 0 ? [] : _ref$rawErrors,
    errors = _ref.errors,
    help = _ref.help,
    description = _ref.description,
    rawDescription = _ref.rawDescription,
    classNames = _ref.classNames,
    style = _ref.style,
    disabled = _ref.disabled,
    label = _ref.label,
    hidden = _ref.hidden,
    onDropPropertyClick = _ref.onDropPropertyClick,
    onKeyChange = _ref.onKeyChange,
    readonly = _ref.readonly,
    required = _ref.required,
    schema = _ref.schema,
    uiSchema = _ref.uiSchema,
    registry = _ref.registry;
  var uiOptions = getUiOptions(uiSchema);
  var WrapIfAdditionalTemplate = getTemplate('WrapIfAdditionalTemplate', registry, uiOptions);
  if (hidden) {
    return jsx("div", {
      className: 'hidden',
      children: children
    });
  }
  return jsx(WrapIfAdditionalTemplate, {
    classNames: classNames,
    style: style,
    disabled: disabled,
    id: id,
    label: label,
    onDropPropertyClick: onDropPropertyClick,
    onKeyChange: onKeyChange,
    readonly: readonly,
    required: required,
    schema: schema,
    uiSchema: uiSchema,
    registry: registry,
    children: jsxs(Form$1.Group, {
      children: [displayLabel && jsxs(Form$1.Label, {
        htmlFor: id,
        className: rawErrors.length > 0 ? 'text-danger' : '',
        children: [label, required ? '*' : null]
      }), children, displayLabel && rawDescription && jsx(Form$1.Text, {
        className: rawErrors.length > 0 ? 'text-danger' : 'text-muted',
        children: description
      }), errors, help]
    })
  });
}

function ObjectFieldTemplate(_ref) {
  var description = _ref.description,
    title = _ref.title,
    properties = _ref.properties,
    required = _ref.required,
    uiSchema = _ref.uiSchema,
    idSchema = _ref.idSchema,
    schema = _ref.schema,
    formData = _ref.formData,
    onAddClick = _ref.onAddClick,
    disabled = _ref.disabled,
    readonly = _ref.readonly,
    registry = _ref.registry;
  var uiOptions = getUiOptions(uiSchema);
  var TitleFieldTemplate = getTemplate('TitleFieldTemplate', registry, uiOptions);
  var DescriptionFieldTemplate = getTemplate('DescriptionFieldTemplate', registry, uiOptions);
  // Button templates are not overridden in the uiSchema
  var AddButton = registry.templates.ButtonTemplates.AddButton;
  return jsxs(Fragment, {
    children: [title && jsx(TitleFieldTemplate, {
      id: titleId(idSchema),
      title: title,
      required: required,
      schema: schema,
      uiSchema: uiSchema,
      registry: registry
    }), description && jsx(DescriptionFieldTemplate, {
      id: descriptionId(idSchema),
      description: description,
      schema: schema,
      uiSchema: uiSchema,
      registry: registry
    }), jsxs(Container, {
      fluid: true,
      className: 'p-0',
      children: [properties.map(function (element, index) {
        return jsx(Row, {
          style: {
            marginBottom: '10px'
          },
          className: element.hidden ? 'd-none' : undefined,
          children: jsxs(Col, {
            xs: 12,
            children: [" ", element.content]
          })
        }, index);
      }), canExpand(schema, uiSchema, formData) ? jsx(Row, {
        children: jsx(Col, {
          xs: {
            offset: 9,
            span: 3
          },
          className: 'py-4',
          children: jsx(AddButton, {
            onClick: onAddClick(schema),
            disabled: disabled || readonly,
            className: 'object-property-expand',
            uiSchema: uiSchema,
            registry: registry
          })
        })
      }) : null]
    })]
  });
}

function SubmitButton(props) {
  var _getSubmitButtonOptio = getSubmitButtonOptions(props.uiSchema),
    submitText = _getSubmitButtonOptio.submitText,
    norender = _getSubmitButtonOptio.norender,
    submitButtonProps = _getSubmitButtonOptio.props;
  if (norender) {
    return null;
  }
  return jsx("div", {
    children: jsx(Button, _extends({
      variant: 'primary',
      type: 'submit'
    }, submitButtonProps, {
      children: submitText
    }))
  });
}

function TitleField(_ref) {
  var id = _ref.id,
    title = _ref.title,
    uiSchema = _ref.uiSchema;
  var uiOptions = getUiOptions(uiSchema);
  return jsxs("div", {
    id: id,
    className: 'my-1',
    children: [jsx("h5", {
      children: uiOptions.title || title
    }), jsx("hr", {
      className: 'border-0 bg-secondary',
      style: {
        height: '1px'
      }
    })]
  });
}

function WrapIfAdditionalTemplate(_ref) {
  var classNames = _ref.classNames,
    style = _ref.style,
    children = _ref.children,
    disabled = _ref.disabled,
    id = _ref.id,
    label = _ref.label,
    onDropPropertyClick = _ref.onDropPropertyClick,
    onKeyChange = _ref.onKeyChange,
    readonly = _ref.readonly,
    required = _ref.required,
    schema = _ref.schema,
    uiSchema = _ref.uiSchema,
    registry = _ref.registry;
  var templates = registry.templates,
    translateString = registry.translateString;
  // Button templates are not overridden in the uiSchema
  var RemoveButton = templates.ButtonTemplates.RemoveButton;
  var keyLabel = translateString(TranslatableString.KeyLabel, [label]);
  var additional = (ADDITIONAL_PROPERTY_FLAG in schema);
  if (!additional) {
    return jsx("div", {
      className: classNames,
      style: style,
      children: children
    });
  }
  var handleBlur = function handleBlur(_ref2) {
    var target = _ref2.target;
    return onKeyChange(target.value);
  };
  var keyId = id + "-key";
  return jsxs(Row, {
    className: classNames,
    style: style,
    children: [jsx(Col, {
      xs: 5,
      children: jsxs(Form$1.Group, {
        children: [jsx(Form$1.Label, {
          htmlFor: keyId,
          children: keyLabel
        }), jsx(Form$1.Control, {
          required: required,
          defaultValue: label,
          disabled: disabled || readonly,
          id: keyId,
          name: keyId,
          onBlur: !readonly ? handleBlur : undefined,
          type: 'text'
        })]
      })
    }), jsx(Col, {
      xs: 5,
      children: children
    }), jsx(Col, {
      xs: 2,
      className: 'py-4',
      children: jsx(RemoveButton, {
        iconType: 'block',
        className: 'w-100',
        disabled: disabled || readonly,
        onClick: onDropPropertyClick(label),
        uiSchema: uiSchema,
        registry: registry
      })
    })]
  }, keyId);
}

function generateTemplates() {
  return {
    ArrayFieldItemTemplate: ArrayFieldItemTemplate,
    ArrayFieldTemplate: ArrayFieldTemplate,
    BaseInputTemplate: BaseInputTemplate,
    ButtonTemplates: {
      AddButton: AddButton,
      CopyButton: CopyButton,
      MoveDownButton: MoveDownButton,
      MoveUpButton: MoveUpButton,
      RemoveButton: RemoveButton,
      SubmitButton: SubmitButton
    },
    DescriptionFieldTemplate: DescriptionField,
    ErrorListTemplate: ErrorList,
    FieldErrorTemplate: FieldErrorTemplate,
    FieldHelpTemplate: FieldHelpTemplate,
    FieldTemplate: FieldTemplate,
    ObjectFieldTemplate: ObjectFieldTemplate,
    TitleFieldTemplate: TitleField,
    WrapIfAdditionalTemplate: WrapIfAdditionalTemplate
  };
}
var Templates = /*#__PURE__*/generateTemplates();

function CheckboxWidget(props) {
  var id = props.id,
    value = props.value,
    disabled = props.disabled,
    readonly = props.readonly,
    label = props.label,
    hideLabel = props.hideLabel,
    schema = props.schema,
    autofocus = props.autofocus,
    options = props.options,
    onChange = props.onChange,
    onBlur = props.onBlur,
    onFocus = props.onFocus,
    registry = props.registry,
    uiSchema = props.uiSchema;
  // Because an unchecked checkbox will cause html5 validation to fail, only add
  // the "required" attribute if the field value must be "true", due to the
  // "const" or "enum" keywords
  var required = schemaRequiresTrueValue(schema);
  var DescriptionFieldTemplate = getTemplate('DescriptionFieldTemplate', registry, options);
  var _onChange = function _onChange(_ref) {
    var checked = _ref.target.checked;
    return onChange(checked);
  };
  var _onBlur = function _onBlur(_ref2) {
    var checked = _ref2.target.checked;
    return onBlur(id, checked);
  };
  var _onFocus = function _onFocus(_ref3) {
    var checked = _ref3.target.checked;
    return onFocus(id, checked);
  };
  var description = options.description || schema.description;
  return jsxs(Form$1.Group, {
    className: "checkbox " + (disabled || readonly ? 'disabled' : ''),
    "aria-describedby": ariaDescribedByIds(id),
    children: [!hideLabel && !!description && jsx(DescriptionFieldTemplate, {
      id: descriptionId(id),
      description: description,
      schema: schema,
      uiSchema: uiSchema,
      registry: registry
    }), jsx(Form$1.Check, {
      id: id,
      name: id,
      label: labelValue(label, hideLabel || !label),
      checked: typeof value === 'undefined' ? false : value,
      required: required,
      disabled: disabled || readonly,
      autoFocus: autofocus,
      onChange: _onChange,
      type: 'checkbox',
      onBlur: _onBlur,
      onFocus: _onFocus
    })]
  });
}

function CheckboxesWidget(_ref) {
  var id = _ref.id,
    disabled = _ref.disabled,
    options = _ref.options,
    value = _ref.value,
    autofocus = _ref.autofocus,
    readonly = _ref.readonly,
    required = _ref.required,
    onChange = _ref.onChange,
    onBlur = _ref.onBlur,
    onFocus = _ref.onFocus;
  var enumOptions = options.enumOptions,
    enumDisabled = options.enumDisabled,
    inline = options.inline,
    emptyValue = options.emptyValue;
  var checkboxesValues = Array.isArray(value) ? value : [value];
  var _onChange = function _onChange(index) {
    return function (_ref2) {
      var checked = _ref2.target.checked;
      if (checked) {
        onChange(enumOptionsSelectValue(index, checkboxesValues, enumOptions));
      } else {
        onChange(enumOptionsDeselectValue(index, checkboxesValues, enumOptions));
      }
    };
  };
  var _onBlur = function _onBlur(_ref3) {
    var value = _ref3.target.value;
    return onBlur(id, enumOptionsValueForIndex(value, enumOptions, emptyValue));
  };
  var _onFocus = function _onFocus(_ref4) {
    var value = _ref4.target.value;
    return onFocus(id, enumOptionsValueForIndex(value, enumOptions, emptyValue));
  };
  return jsx(Form$1.Group, {
    children: Array.isArray(enumOptions) && enumOptions.map(function (option, index) {
      var checked = enumOptionsIsSelected(option.value, checkboxesValues);
      var itemDisabled = Array.isArray(enumDisabled) && enumDisabled.indexOf(option.value) !== -1;
      return jsx(Form$1.Check, {
        inline: inline,
        custom: true,
        required: required,
        checked: checked,
        className: 'bg-transparent border-0',
        type: 'checkbox',
        id: optionId(id, index),
        name: id,
        label: option.label,
        autoFocus: autofocus && index === 0,
        onChange: _onChange(index),
        onBlur: _onBlur,
        onFocus: _onFocus,
        disabled: disabled || itemDisabled || readonly,
        "aria-describedby": ariaDescribedByIds(id)
      }, option.value);
    })
  });
}

function RadioWidget(_ref) {
  var id = _ref.id,
    options = _ref.options,
    value = _ref.value,
    required = _ref.required,
    disabled = _ref.disabled,
    readonly = _ref.readonly,
    onChange = _ref.onChange,
    onBlur = _ref.onBlur,
    onFocus = _ref.onFocus;
  var enumOptions = options.enumOptions,
    enumDisabled = options.enumDisabled,
    emptyValue = options.emptyValue;
  var _onChange = function _onChange(_ref2) {
    var value = _ref2.target.value;
    return onChange(enumOptionsValueForIndex(value, enumOptions, emptyValue));
  };
  var _onBlur = function _onBlur(_ref3) {
    var value = _ref3.target.value;
    return onBlur(id, enumOptionsValueForIndex(value, enumOptions, emptyValue));
  };
  var _onFocus = function _onFocus(_ref4) {
    var value = _ref4.target.value;
    return onFocus(id, enumOptionsValueForIndex(value, enumOptions, emptyValue));
  };
  var inline = Boolean(options && options.inline);
  return jsx(Form$1.Group, {
    className: 'mb-0',
    children: Array.isArray(enumOptions) && enumOptions.map(function (option, index) {
      var itemDisabled = Array.isArray(enumDisabled) && enumDisabled.indexOf(option.value) !== -1;
      var checked = enumOptionsIsSelected(option.value, value);
      var radio = jsx(Form$1.Check, {
        inline: inline,
        label: option.label,
        id: optionId(id, index),
        name: id,
        type: 'radio',
        disabled: disabled || itemDisabled || readonly,
        checked: checked,
        required: required,
        value: String(index),
        onChange: _onChange,
        onBlur: _onBlur,
        onFocus: _onFocus,
        "aria-describedby": ariaDescribedByIds(id)
      }, index);
      return radio;
    })
  });
}

function RangeWidget(props) {
  var value = props.value,
    label = props.label,
    hideLabel = props.hideLabel,
    options = props.options,
    registry = props.registry;
  var BaseInputTemplate = getTemplate('BaseInputTemplate', registry, options);
  return jsx(BaseInputTemplate, _extends({}, props, {
    extraProps: {
      label: labelValue(label || undefined, hideLabel)
    },
    children: jsx("span", {
      className: 'range-view',
      children: value
    })
  }));
}

function SelectWidget(_ref) {
  var schema = _ref.schema,
    id = _ref.id,
    options = _ref.options,
    required = _ref.required,
    disabled = _ref.disabled,
    readonly = _ref.readonly,
    value = _ref.value,
    multiple = _ref.multiple,
    autofocus = _ref.autofocus,
    _onChange = _ref.onChange,
    onBlur = _ref.onBlur,
    onFocus = _ref.onFocus,
    placeholder = _ref.placeholder,
    _ref$rawErrors = _ref.rawErrors,
    rawErrors = _ref$rawErrors === void 0 ? [] : _ref$rawErrors;
  var enumOptions = options.enumOptions,
    enumDisabled = options.enumDisabled,
    optEmptyValue = options.emptyValue;
  var emptyValue = multiple ? [] : '';
  function getValue(event, multiple) {
    if (multiple) {
      return [].slice.call(event.target.options).filter(function (o) {
        return o.selected;
      }).map(function (o) {
        return o.value;
      });
    } else {
      return event.target.value;
    }
  }
  var selectedIndexes = enumOptionsIndexForValue(value, enumOptions, multiple);
  return jsxs(Form$1.Control, {
    as: 'select',
    bsPrefix: 'custom-select',
    id: id,
    name: id,
    value: typeof selectedIndexes === 'undefined' ? emptyValue : selectedIndexes,
    required: required,
    multiple: multiple,
    disabled: disabled || readonly,
    autoFocus: autofocus,
    className: rawErrors.length > 0 ? 'is-invalid' : '',
    onBlur: onBlur && function (event) {
      var newValue = getValue(event, multiple);
      onBlur(id, enumOptionsValueForIndex(newValue, enumOptions, optEmptyValue));
    },
    onFocus: onFocus && function (event) {
      var newValue = getValue(event, multiple);
      onFocus(id, enumOptionsValueForIndex(newValue, enumOptions, optEmptyValue));
    },
    onChange: function onChange(event) {
      var newValue = getValue(event, multiple);
      _onChange(enumOptionsValueForIndex(newValue, enumOptions, optEmptyValue));
    },
    "aria-describedby": ariaDescribedByIds(id),
    children: [!multiple && schema["default"] === undefined && jsx("option", {
      value: '',
      children: placeholder
    }), enumOptions.map(function (_ref2, i) {
      var value = _ref2.value,
        label = _ref2.label;
      var disabled = Array.isArray(enumDisabled) && enumDisabled.indexOf(value) != -1;
      return jsx("option", {
        id: label,
        value: String(i),
        disabled: disabled,
        children: label
      }, i);
    })]
  });
}

function TextareaWidget(_ref) {
  var id = _ref.id,
    placeholder = _ref.placeholder,
    value = _ref.value,
    required = _ref.required,
    disabled = _ref.disabled,
    autofocus = _ref.autofocus,
    readonly = _ref.readonly,
    onBlur = _ref.onBlur,
    onFocus = _ref.onFocus,
    onChange = _ref.onChange,
    options = _ref.options;
  var _onChange = function _onChange(_ref2) {
    var value = _ref2.target.value;
    return onChange(value === '' ? options.emptyValue : value);
  };
  var _onBlur = function _onBlur(_ref3) {
    var value = _ref3.target.value;
    return onBlur(id, value);
  };
  var _onFocus = function _onFocus(_ref4) {
    var value = _ref4.target.value;
    return onFocus(id, value);
  };
  return jsx(InputGroup, {
    children: jsx(FormControl, {
      id: id,
      name: id,
      as: 'textarea',
      placeholder: placeholder,
      disabled: disabled,
      readOnly: readonly,
      value: value,
      required: required,
      autoFocus: autofocus,
      rows: options.rows || 5,
      onChange: _onChange,
      onBlur: _onBlur,
      onFocus: _onFocus,
      "aria-describedby": ariaDescribedByIds(id)
    })
  });
}

function generateWidgets() {
  return {
    CheckboxWidget: CheckboxWidget,
    CheckboxesWidget: CheckboxesWidget,
    RadioWidget: RadioWidget,
    RangeWidget: RangeWidget,
    SelectWidget: SelectWidget,
    TextareaWidget: TextareaWidget
  };
}
var Widgets = /*#__PURE__*/generateWidgets();

function generateTheme() {
  return {
    templates: generateTemplates(),
    widgets: generateWidgets()
  };
}
var Theme = /*#__PURE__*/generateTheme();

function generateForm() {
  return withTheme(generateTheme());
}
var Form = /*#__PURE__*/generateForm();

export { Form, Templates, Theme, Widgets, Form as default, generateForm, generateTemplates, generateTheme, generateWidgets };
//# sourceMappingURL=bootstrap-4.esm.js.map
